
/**
* The main source code for the pager. Has not been minified and contains all comments.
* @file source/main.js
* @author marco
* @since 2/16/12
* @version 0.1
*/

/**
* A jQuery pager module. It's called on a container html element which holds the items to be paged. It will go through the items and setup the pager.
* It's not really a typical pager as pagination typically has pages with page numbers. The move pager instead does not setup pages
* but uses an itemsMoveAmount variable (which is a number) which controls how many items should be moved forward and back when the previous and next buttons
* are clicked. It will also show a certain number of items, even when reaching the beginning and end. These is specifed by the itemsPerPage in the options
* object.
 * <br />
 * <br />
* The pager does not add any classes or ids to the html elements except for the "active" class for both the previous and next buttons. This class is toggled
* on/off for when the end of the pager is reached and can be set manually through a public interface. A publically accessible array is used to keep track of
* the items in the pager. This array can be modified after the pager has been initialized by either using the update() method or appending to the pager div
* and calling the update() function with no parameters. New elements are set to hidden by the pager and should automatically be set to hidden by the user.
* <br />
 * <br />
* The pager object itself is publically accessible and can be modified to suit people's needs. Most of it's functions are also accessible and can be modified.
* It uses the jQuery plugin object pattern for public access where you set the data on the element being page to 'movePager'. This can be done like this:
 *<br />
 * <br />
* var pager = $('#page').movePager(options2).data('movePager');
 * <br />
* or
 * <br />
* $('#page').movePager;
* var pager = $('#page').data('movePager');
*<br />
 *<br />
* It used this pattern: http://stefangabos.ro/jquery/jquery-plugin-boilerplate-revisited/.
*
* @project ObjectPager
* @author marco
* @since 2/14/12
* @version 0.1
*/

/**
* A pager which uses move amount to move to new pages rather then page numbers. Requires jQuery.
* @see ObjectPager
* @module ObjectPager
* @author marco
* @version 0.1
*/

(function( $ ) {

    /**
    * The jQuery namespace, aliases as $, which the pager will be under. Once institated, the pager can be extended by accessing it publicly
    * using this namespace.
    * @namespace $
    */

    /**
    * The main pager class, which is initialized by the $.fn.pager function.
    * @see $.fn.pager
    * @class {public void} $.ObjectPager
    */
    /**
    * The constructor for the ObjectPager class. This is called by $.fn.pager which can be called on any element in an HTML document with jQuery.
    * @constructor {ObjectPager}  ObjectPager
    * @param {Object} element - The jQuery node that this pager will be created on. This is automatically set by $.fn.pager.
    * @param {Array} data - an array of jQuery objects that should be used in the pager. This is not required, the elements within the pager element
    * will be used otherwise.
    * @param {Object} options - an object specifying all the options @see $.ObjectPager.settings
    */
    $.ObjectPager = function(element, options, data) {
        /**
        * Default options object, which is overridden with the options supplied by user
        * @see $.ObjectPager.settings
        * @var {private Object} defaults
        */
        var defaults = {
            itemsPerPage: 3,
            itemsMoveAmount: 3,
            wrapAround: false,
            prevButton: $('<a href="#" id="prev">Previous</a>') ,
            nextButton: $('<a href="#" id="next">Next</a>'),
            startPage: 0
        };

        /**
         * This variable essentially sets which page will be shown when the pager is initialized. It is the first item that should be showing. It is then
         * used by the paging function as a reference to what items is the first one showing. It is publically accessible with $.ObjectPager.getCurrentFirstItem
         * @see $.ObjectPager.getCurrentFirstItem
         * @var {private Number} p_firstItemShowing
         */
        var p_firstItemShowing;

        /**
        * Variable that stores the this keyword for the main pager object, fixes javascript error with inner private methods
        * @var {private Object} pager
        */
        var pager = this;
        /**
        * The variable that holds the jQuery object that this pager was called on and was passed for the $.fn init method
        * @var {private Object} $this
        */
        var $this = $(element);


        /**
        * The button object which will provide additional functionality for the jQuery buttons. They are extended to the jQuery ojects. This has been
        * been created using the Factory pattern so that a new object doesn't need to be created but is created within the button function..
        * @see $.ObjectPager.$prevButton
        * @see $.ObjectPager.$nextButton
        * @object {private} $.ObjectPager.button
        */
        var button = (function() {
            var button = function() {
                /**
                 * The active variable which specifies whether the current button is active or not, Has getters and setters.
                 * @property {private Boolean }  $.ObjectPager.button.active
                 */
                var active;
                /**
                 * This method returns whether the button is active or not. It checks the private active variable.
                 * @function {public Boolean} $.ObjectPager.button.isActive
                 * @returns The inner variable which specifies whether the object is active or not
                 */
                this.isActive =  function() {
                    return active;
                };
                /**
                 * This method sets the button to active or inactive. It changes the active variable and sets the class correctly. It does a quick check to see
                 * if it's a boolean, this hasn't been tested though. It is used internally by the move and show functions. It also will automatically set active and the class to true if wrapAround is strue. This is
                 * strange functionality as this method will no longer work correctly when wrapAround is true, but it makes sure the pager is not broken.
                 * @function {public void} $.ObjectPager.button.setActive
                 * @param {Boolean} bool - A true Boolean value (not a string) on whether the button should be set to active or not active.
                 * @throws $.error if the param is not a Boolean
                 */
                this.setActive =  function(bool) {
                    if (typeof bool === 'boolean') {
                        if (!pager.settings.wrapAround) {
                            active = bool;
                            this.toggleClass('active', bool);
                        }
                        else {
                            active = true;
                            this.toggleClass('active', true);
                        }
                    }
                    else { $.error('Error: you need to specify a boolean to set the buttons to active or not.'); }
                };
            };
            return function() {
                return new button();
            };

        })();

        // public variables:
        /**
        * An object property to hold the jQuery object for the prev button element. This has a click event listener attached to it to handle prev and next. This listener can technically
        * be replaced or even added on to. The button is publically accessible so you can add another even listener to it, say for hover, and call the pager prev or next functions
        * on the correct event.
        * @see $.ObjectPager.prev
        * @object {public } $.ObjectPager.$prevButton
        * @extends $.ObjectPager.button
        */
        pager.$prevButton = {};
        /**
         * An object property to hold the jQuery object for the next button element. This has a click event listener attached to it to handle prev and next. This listener can technically
         * be replaced or even added on to. The button is publically accessible so you can add another even listener to it, say for hover, and call the pager prev or next functions
         * on the correct event.
         * @see $.ObjectPager.next
         * @object {public } $.ObjectPager.$nextButton
         * @extends $.ObjectPager.button
         */
        pager.$nextButton = {};
        /**
        * An array holding all the jQuery objects that will be paged. This is used by the whole class to flip through the pages of items. It is modifed with the
        * pager.update() function.
        * @var {public Array} $.ObjectPager.$items
        */
        pager.$items = [];


        /**
        * Initialiation method for pager, used mainly to make the code a little easier to read. It overwirtes the settings object
        * with the options passed in by the user. It then sets the private prev, next button variables and binds a click event to the
        * pager.prev and next methods. It loops through any passed in data and creates jQuery items, adding them to the private items array. There is error
        * checking done before the pager's button's are binded or it calls show() on the correct pages. This is checking to make sure that the
        * total number of items is more then the itemsPerPage or itemsMove settings. The buttons are checked to see if they are real jQuery objects. It
        * also checks to make sure the startPage is within the items. It uses the first page variable to set whether the prev and next buttons should be active or not.
        * @function {private void} $.ObjectPager.init
        * @throws $.error when either the prev or next buttons are not jQuery objects
        * @throws $.error when there are not enough elements to create a pager, based on itemTotal and itemsPerPage
        */
        var init = function() {

            /**
            * This settings objects is extended from the private default object so any values the user specifies during init are changed. This settings object
            * will be used by the application to get the correct values.
            * @object {public Object} $.ObjectPager.settings
            */

            /**
            * The amount of items that will be showing at all times. This should always be more or equal to the move amount. It should not be greater then the total amount,
            * or there would be nothing to show.
            *
            * Defaulted to 5.
            * @property {Number} $.ObjectPager.settings.itemsPerPage
            */
            /**
            * This is the amount, defaulted to 5 and set by user, that the items will move on the page. This is
            * different then items per page as you can move less items per page then necessary
            *
            * Defaulted to 5.
            * @property {Number} $.ObjectPager.settings.itemsMoveAmount
            */
            /**
            * The wrapAround values specifies whether the pager should wrap around to the beginning/end when it reaches the end/beginning of the items. It is used by the
            * paging functions
            *
            * Defaulted to false.
            * @property {Boolean} $.ObjectPager.settings.wrapAround
            */
            /**
            * The startPage is the page that the pager will start on when first initialized. It is not actually a page, but rather the first item that should be showing. Will not be
            * used after pager initialization.
            *
            * Defaulted to 0.
            * @property {Number} $.ObjectPager.settings.startPage
            */
            pager.settings = $.extend(defaults, options);
            var j;		
		
            // jQuery items have to just add it to items
            $.each($this.children(), function() {
                pager.$items.push($(this)[0]);
            });

            if(data !== undefined && data.length > 0) {
                for (j=0; j<data.length; j++) {
                    try {
                        pager.$items[j] = $(data[j]);
                        $this.append(pager.$items[j]);
                    }
                    catch(err) {
                        $.error('The array of objects you passed in as data cannot be created as a jQuery object: ' +err+ "\n");
                    }
                }
            }

            if (pager.$items.length < pager.settings.itemsPerPage || pager.$items.length < pager.settings.itemsMoveAmount) {
                $.error("There aren't enough elements in the pager to be used with your items-per-page, item-move-amount options");
                return;
            }

            // in case prev/next buttons weren't given this will append them before and after the main pager div
            if (options.prevButton === undefined) {
                $this.before(pager.settings.prevButton);
            }
            if (options.nextButton === undefined) {
                $this.after(pager.settings.nextButton);
            }
            // code to look though options and pull out paging buttons
            pager.$prevButton = pager.settings.prevButton.bind('click', pager.prev);
            pager.$nextButton = pager.settings.nextButton.bind('click', pager.next);
            if (pager.settings.prevButton.length < 1 || pager.settings.nextButton < 1) {
                $.error("The jQuery objects for one of your prev/next buttons is not a jQuery object");
                return;
            }

            $.extend(pager.$prevButton, button());
            $.extend(pager.$nextButton, button());

            pager.show(pager.settings.startPage);

        };

        /**
        * A function not available to outside plugin user, this move function takes the first and next items to be shown. It slices the
        * item array at the first +/- itemsPerPage amount to hide the items and does the same with the next +/- itemsPerPage to show the
        * items. This function does not care whether the pager is moving back or forward, it just takes the first and next and displays the new page.
        * Also sets the firstItem vairable to the new first item after updating. It also sets the prev/next buttons correctly to off if it's at a boundary
        * @function {private Boolean} $.ObjectPager.move
        * @param {Number} first - The first item that is currently showing
        * @param {Number} next - The first item that needs to be showing after the move is complete
        * @returns true if successful, false if unsucessful
        * @throws $.error if the next is less the 0 or the next is more then the item total, meaning a move that cannot be made
        */
        var move = function(first, next) {
            // function gets first element and the next first element and needs to hide() and show()
            if (Math.abs(first - next) > pager.settings.itemsPerPage) {
                $.error("Warning: your item-move-amount option is more then the items-per-page option. This will mean items will be skipped over when paging through");
            }
            if (next < 0 || next > pager.$items.length) {
                $.error("you are trying to move to a page that does not exist.");
                return false;
            }

            $.each(pager.$items.slice(first, first + pager.settings.itemsPerPage), function() { $(this).hide(); });
            $.each(pager.$items.slice(next, next + pager.settings.itemsPerPage), function() { $(this).show(); } );

            pager.$nextButton.setActive(true);
            if(next + pager.settings.itemsPerPage >= pager.$items.length) {
                pager.$nextButton.setActive(false);
            }

            pager.$prevButton.setActive(true);
            if (next <= 0) {
                pager.$prevButton.setActive(false);
            }

            p_firstItemShowing = next;
            return true;
        };

        // public:
        /**
        * The show method which takes the first element and then uses the itemsPerPage to show a sequence of items. This is
        * not used for the prev and next functions because it iterates through all items, setting them to hide before operating.
        * It then shows by slicing an array from first and last. Displays an error if the page being requested to show is more or less
        * then the available items. At the end it will make checks to see if the buttons should be active or not.
        * @function {public Boolean} $.ObjectPager.show
        * @param {Number} first - The first item in the next page that will be shown.
        * @returns Returns false if an error condition is met, otherwise true
        */
        pager.show = function(first) {
            if (first > pager.$items.length || first < 0) {
                $.error("You are attempting to show a page which does not exist");
                return false;
            }

            if (first + pager.settings.itemsPerPage > pager.$items.length) {
                $.error("You are attempting to show a page which will not allow to show the full amount of pages becuase its too high on the total list of items");
                return false;
            }

            // TODO make this seperate function to allow different animations
            $.each(pager.$items, function() { $(this).hide(); });
            $.each(pager.$items.slice(first, first + pager.settings.itemsPerPage), function() { $(this).show(); });

            pager.$nextButton.setActive(true);
            if(first + pager.settings.itemsPerPage >= pager.$items.length) {
                pager.$nextButton.setActive(false);
            }

            pager.$prevButton.setActive(true);
            if (first <= 0) { pager.$prevButton.setActive(false); }

            p_firstItemShowing = first;
            return true;
        };

        /**
        * A method to update the pager with new items. Takes either an array of jQuery objects or just one jQuery object, adds them to items, appends them to the
        * end of the current element the pager is on and then sets the total items. It can be called with no data and it will just update by showing the current page.
        * Returns all the items in an array. Will also automatically hide the item at first but will be unhidden if its on the correct page.
        * @function {pubilc Array} $.ObjectPager.update
        * @param {Array} data - An array of jQuery objects or just on jQuery object
        * @return {Array} $.ObjectPager.$items - returns all the items after the update has been completed
        */
        pager.update = function(data) {
            var i;
            if (data !== undefined && data.length > 0) {
                for (i=0; i<data.length; i++) {
                    pager.$items.push($(data[i]));
                    $this.append($(data[i]));
                    // assunmes you want the data hidden
                    $(data[i]).hide();
                }
            }
            pager.show(p_firstItemShowing);
            return pager.$items;
        };

        /**
         * The public prev function which first checks to see if it's button is active, then sets what the next first element is,
         * and will call the move method with the first and next numbers. If next is below zero, or it's trying to move to an item below
         * zero then it will either wrap around to the top or show the first item along with however many items are per page. It will automatically
         * set the next button to active and toggles it's active class.
        * @function {public void} $.ObjectPager.prev
        * @param {Event} e
        */
        pager.prev = function(e) {
            if (pager.$prevButton.isActive()) {
                // this function will just check which pages have to be moved and then pass off to another move function, passed only first item on current page
                var next = p_firstItemShowing - pager.settings.itemsMoveAmount;

                if (next >= 0) {
                    move(p_firstItemShowing, next);
                }
                else {
                    // trying to move to a page that is less then 0, so move only to 0, or wrap around depening on settings
                    if (pager.settings.wrapAround) {
                        pager.show(pager.$items.length - pager.settings.itemsPerPage);
                    }
                    else {
                        pager.show(0);

                    }
                }

            }
            if (e !== undefined) {
                e.preventDefault();
            }
        };

        /**
         * The public prev function which first checks to see if it's button is active, then sets what the next first element is,
         * and will call the move method with the first and next numbers. If next is below zero, or it's trying to move to an item below
         * zero then it will either wrap around to the top or show the first item along with however many items are per page. It will automatically
         * set the next button to active and toggles it's active class.
         * @function {public void} $.ObjectPager.next
         * @param {Event} e
         */
        pager.next = function(e) {
            if (pager.$nextButton.isActive()) {
                var next = p_firstItemShowing + pager.settings.itemsMoveAmount;

                if (next + pager.settings.itemsPerPage <= pager.$items.length) {
                    move(p_firstItemShowing, next);
                }
                else {
                    if (pager.settings.wrapAround) {
                        pager.show(0);
                    }
                    else {
                        pager.show(pager.$items.length - pager.settings.itemsPerPage);
                    }
                }
            }
            if (e !== undefined) {
                e.preventDefault();
            }
        };

        /**
        * Will return the first item that is currently showing. A public interface to check this private variable
        * @function {public Number} $.ObjectPager.getCurrentFirstItem
        * @see  p_firstItemShowing
        * @returns p_firstItemShowing
        */
        pager.getCurrentFirstItem = function() {
            return p_firstItemShowing;
        };


        init();
    };

    /**
     * The jQuery prototype namespace, $.fn where fn is a shortcut to jQuery's prototype. This is where the pager is initialized.
     * @namespace $.fn
     */

    /**
    * The function that is public and called by the user to initilize and after modify the pager.
    * @function {public Object} $.fn.pager
    * @param {Object} options
    * @param {Array} data
    * @returns Returns a new ObjectPager object
    */
    $.fn.movePager = function(options, data) {

        return this.each(function() {
            if ($(this).data('movePager')  === undefined) {
                var pager = new $.ObjectPager(this, options, data);
                $(this).data('movePager', pager);
                return this;
            }
        });
    };

})( jQuery );



